@@ -1,12 +1,13 @@
 import asyncio
 import os
-import sys
-import ssl
 import socket
-import types
+import ssl
+import sys
+import typing
 import warnings
+from concurrent.futures import CancelledError
 from io import BytesIO
-from aredis.utils import b, nativestr
+
 from aredis.exceptions import (ConnectionError, TimeoutError,
                                RedisError, ExecAbortError,
                                BusyLoadingError, NoScriptError,
@@ -14,6 +15,7 @@ from aredis.exceptions import (ConnectionError, TimeoutError,
                                InvalidResponse, AskError,
                                MovedError, TryAgainError,
                                ClusterDownError, ClusterCrossSlotError)
+from aredis.utils import b, nativestr
 
 try:
     import hiredis
@@ -194,6 +196,8 @@ class PythonParser(BaseParser):
         return self._buffer and bool(self._buffer.length)
 
     async def read_response(self):
+        if not self._buffer:
+            raise ConnectionError('Socket closed on remote end')
         response = await self._buffer.readline()
         if not response:
             raise ConnectionError('Socket closed on remote end')
@@ -297,10 +301,13 @@ class HiredisParser(BaseParser):
         while response is False:
             try:
                 buffer = await self._stream.read(self._read_size)
+            # CancelledError will be caught by client so that command won't be retried again
+            # For more detailed discussion please see https://github.com/NoneGG/aredis/issues/56
+            except CancelledError:
+                raise
             except Exception:
                 e = sys.exc_info()[1]
-                raise ConnectionError("Error while reading from stream: %s" %
-                                      (e.args,))
+                raise ConnectionError("Error {} while reading from stream: {}".format(type(e), e.args))
             if not buffer:
                 raise ConnectionError("Socket closed on remote end")
             self._reader.feed(buffer)
@@ -369,6 +376,8 @@ class BaseConnection:
         self.encoding = encoding
         self.decode_responses = decode_responses
         self.loop = loop
+        # flag to show if a connection is waiting for response
+        self.awaiting_response = False
 
     def __repr__(self):
         return self.description.format(**self._description_args)
@@ -400,7 +409,7 @@ class BaseConnection:
         # is for pubsub channel/pattern resubscription
         for callback in self._connect_callbacks:
             task = callback(self)
-            if isinstance(task, types.CoroutineType):
+            if isinstance(task, typing.Awaitable):
                 await task
 
     async def _connect(self):
@@ -429,6 +438,7 @@ class BaseConnection:
             raise
         if isinstance(response, RedisError):
             raise response
+        self.awaiting_response = False
         return response
 
     async def send_packed_command(self, command):
@@ -460,6 +470,7 @@ class BaseConnection:
         if not (self._reader and self._writer):
             await self.connect()
         await self.send_packed_command(self.pack_command(*args))
+        self.awaiting_response = True
 
     def encode(self, value):
         "Return a bytestring representation of the value"
@@ -470,7 +481,7 @@ class BaseConnection:
         elif isinstance(value, float):
             value = b(repr(value))
         elif not isinstance(value, str):
-            value = str(value).encode()
+            value = str(value).encode(self.encoding)
         return value
 
     def disconnect(self):
