@@ -1,17 +1,14 @@
 import sys
-import types
+import typing
 from itertools import chain
+
 from aredis.client import (StrictRedisCluster, StrictRedis)
-from aredis.exceptions import (RedisError,
-                               ConnectionError,
-                               TimeoutError,
-                               ResponseError,
-                               WatchError,
-                               ExecAbortError,
-                               MovedError,
-                               AskError,
-                               TryAgainError,
-                               RedisClusterException)
+from aredis.exceptions import (RedisError, ConnectionError,
+                               TimeoutError, ResponseError,
+                               WatchError, ExecAbortError,
+                               MovedError, AskError,
+                               TryAgainError, RedisClusterException,
+                               ClusterTransactionError)
 from aredis.utils import (dict_merge,
                           clusterdown_wrapper)
 
@@ -202,10 +199,9 @@ class BasePipeline(object):
                 command_name = args[0]
                 if command_name in self.response_callbacks:
                     callback = self.response_callbacks[command_name]
-                    if isinstance(callback, types.CoroutineType):
-                        r = await callback(r, **options)
-                    else:
-                        r = callback(r, **options)
+                    r = callback(r, **options)
+                    if isinstance(response, typing.Awaitable):
+                        r = await r
             data.append(r)
         return data
 
@@ -261,12 +257,11 @@ class BasePipeline(object):
         shas = [s.sha for s in scripts]
         # we can't use the normal script_* methods because they would just
         # get buffered in the pipeline.
-        exists = await immediate('SCRIPT', 'EXISTS', *shas, **{'parse': 'EXISTS'})
+        exists = await immediate('SCRIPT EXISTS', *shas)
         if not all(exists):
             for s, exist in zip(scripts, exists):
                 if not exist:
-                    s.sha = await immediate('SCRIPT', 'LOAD', s.script,
-                                            **{'parse': 'LOAD'})
+                    s.sha = await immediate('SCRIPT LOAD', s.script)
 
     async def execute(self, raise_on_error=True):
         "Execute all the commands in the current pipeline"
@@ -276,9 +271,9 @@ class BasePipeline(object):
         if self.scripts:
             await self.load_scripts()
         if self.transaction or self.explicit_transaction:
-            execute = self._execute_transaction
+            exec = self._execute_transaction
         else:
-            execute = self._execute_pipeline
+            exec = self._execute_pipeline
 
         conn = self.connection
         if not conn:
@@ -288,7 +283,7 @@ class BasePipeline(object):
             self.connection = conn
 
         try:
-            return await execute(conn, stack, raise_on_error)
+            return await exec(conn, stack, raise_on_error)
         except (ConnectionError, TimeoutError) as e:
             conn.disconnect()
             if not conn.retry_on_timeout and isinstance(e, TimeoutError):
@@ -303,7 +298,7 @@ class BasePipeline(object):
                                  "one or more keys")
             # otherwise, it's safe to retry since the transaction isn't
             # predicated on any state
-            return await execute(conn, stack, raise_on_error)
+            return await exec(conn, stack, raise_on_error)
         finally:
             await self.reset()
 
@@ -317,16 +312,6 @@ class BasePipeline(object):
         "Unwatches all previously specified keys"
         return self.watching and await self.execute_command('UNWATCH') or True
 
-    async def script_load_for_pipeline(self, script):
-        "Make sure scripts are loaded prior to pipeline execution"
-        # we need the sha now so that Script.__call__ can use it to run
-        # evalsha.
-        if not script.sha:
-            script.sha = await self.immediate_execute_command('SCRIPT', 'LOAD',
-                                                              script.script,
-                                                              **{'parse': 'LOAD'})
-        self.scripts.add(script)
-
 
 class StrictPipeline(BasePipeline, StrictRedis):
     "Pipeline for the StrictRedis class"
@@ -335,9 +320,8 @@ class StrictPipeline(BasePipeline, StrictRedis):
 
 class StrictClusterPipeline(StrictRedisCluster):
     def __init__(self, connection_pool, result_callbacks=None,
-                 response_callbacks=None, startup_nodes=None):
-        """
-        """
+                 response_callbacks=None, startup_nodes=None,
+                 transaction=False, watches=None):
         self.command_stack = []
         self.refresh_table_asap = False
         self.connection_pool = connection_pool
@@ -345,22 +329,16 @@ class StrictClusterPipeline(StrictRedisCluster):
         self.startup_nodes = startup_nodes if startup_nodes else []
         self.nodes_flags = self.__class__.NODES_FLAGS.copy()
         self.response_callbacks = dict_merge(response_callbacks or self.__class__.RESPONSE_CALLBACKS.copy())
+        self.transaction = transaction
+        self.watches = watches or None
+        self.watching = False
+        self.explicit_transaction = False
 
     def __repr__(self):
         """
         """
         return "{0}".format(type(self).__name__)
 
-    def __enter__(self):
-        """
-        """
-        return self
-
-    def __exit__(self, exc_type, exc_value, traceback):
-        """
-        """
-        self.reset()
-
     def __del__(self):
         """
         """
@@ -371,6 +349,12 @@ class StrictClusterPipeline(StrictRedisCluster):
         """
         return len(self.command_stack)
 
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, exc_type, exc_val, exc_tb):
+        self.reset()
+
     def _determine_slot(self, *args):
         """
         figure out what slot based on command and args
@@ -417,9 +401,12 @@ class StrictClusterPipeline(StrictRedisCluster):
 
         if not stack:
             return []
-
+        if self.transaction:
+            execute = self.send_cluster_transaction
+        else:
+            execute = self.send_cluster_commands
         try:
-            return await self.send_cluster_commands(stack, raise_on_error)
+            return await execute(stack, raise_on_error)
         finally:
             self.reset()
 
@@ -430,30 +417,61 @@ class StrictClusterPipeline(StrictRedisCluster):
         self.command_stack = []
 
         self.scripts = set()
-
-        # TODO: Implement
-        # make sure to reset the connection state in the event that we were
-        # watching something
-        # if self.watching and self.connection:
-        #     try:
-        #         # call this manually since our unwatch or
-        #         # immediate_execute_command methods can call reset()
-        #         self.connection.send_command('UNWATCH')
-        #         self.connection.read_response()
-        #     except ConnectionError:
-        #         # disconnect will also remove any previous WATCHes
-        #         self.connection.disconnect()
-
+        self.watches = []
         # clean up the other instance attributes
         self.watching = False
         self.explicit_transaction = False
 
-        # TODO: Implement
-        # we can safely return the connection to the pool here since we're
-        # sure we're no longer WATCHing anything
-        # if self.connection:
-        #     self.connection_pool.release(self.connection)
-        #     self.connection = None
+    @clusterdown_wrapper
+    async def send_cluster_transaction(self, stack, raise_on_error=True):
+        # the first time sending the commands we send all of the commands that were queued up.
+        # if we have to run through it again, we only retry the commands that failed.
+        attempt = sorted(stack, key=lambda x: x.position)
+        node = {}
+
+        # as we move through each command that still needs to be processed,
+        # we figure out the slot number that command maps to, then from the slot determine the node.
+        for c in attempt:
+            # refer to our internal node -> slot table that tells us where a given
+            # command should route to.
+            slot = self._determine_slot(*c.args)
+            hashed_node = self.connection_pool.get_node_by_slot(slot)
+
+            # now that we know the name of the node ( it's just a string in the form of host:port )
+            # we can build a list of commands for each node.
+            if node.get('name') != hashed_node['name']:
+                # raise error if commands in a transaction can not hash to same node
+                if len(node) > 0:
+                    raise ClusterTransactionError("Keys in request don't hash to the same node")
+            node = hashed_node
+        conn = self.connection_pool.get_connection_by_node(node)
+        if self.watches:
+            await self._watch(node, conn, self.watches)
+        node_commands = NodeCommands(self.parse_response, conn, in_transaction=True)
+        node_commands.append(PipelineCommand(('MULTI',)))
+        node_commands.extend(attempt)
+        node_commands.append(PipelineCommand(('EXEC',)))
+        self.explicit_transaction = True
+        await node_commands.write()
+        # todo: make this place clear
+        try:
+            await node_commands.read()
+        except ExecAbortError:
+            if self.explicit_transaction:
+                await conn.send_command('DISCARD')
+                await conn.read_response()
+
+        # If at least one watched key is modified before the EXEC command,
+        # the whole transaction aborts,
+        # and EXEC returns a Null reply to notify that the transaction failed.
+        if node_commands.commands[-1].result is None:
+            raise WatchError
+        self.connection_pool.release(conn)
+        if self.watching:
+            self._unwatch(conn)
+        if raise_on_error:
+            self.raise_first_error(stack)
+
 
     @clusterdown_wrapper
     async def send_cluster_commands(self, stack, raise_on_error=True, allow_redirections=True):
@@ -560,35 +578,38 @@ class StrictClusterPipeline(StrictRedisCluster):
         if not allow_redirections:
             raise RedisClusterException("ASK & MOVED redirection not allowed in this pipeline")
 
-    def multi(self):
+    def _multi(self):
         """
         """
         raise RedisClusterException("method multi() is not implemented")
 
     def immediate_execute_command(self, *args, **options):
-        """
-        """
         raise RedisClusterException("method immediate_execute_command() is not implemented")
 
-    def _execute_transaction(self, *args, **kwargs):
-        """
-        """
-        raise RedisClusterException("method _execute_transaction() is not implemented")
-
     def load_scripts(self):
         """
         """
         raise RedisClusterException("method load_scripts() is not implemented")
 
-    def watch(self, *names):
-        """
-        """
-        raise RedisClusterException("method watch() is not implemented")
+    async def _watch(self, node, conn, names):
+        "Watches the values at keys ``names``"
+        for name in names:
+            slot = self._determine_slot('WATCH', name)
+            dist_node = self.connection_pool.get_node_by_slot(slot)
+            if node.get('name') != dist_node['name']:
+                # raise error if commands in a transaction can not hash to same node
+                if len(node) > 0:
+                    raise ClusterTransactionError("Keys in request don't hash to the same node")
+        if self.explicit_transaction:
+            raise RedisError('Cannot issue a WATCH after a MULTI')
+        await conn.send_command('WATCH', *names)
+        return await conn.read_response()
 
-    def unwatch(self):
-        """
-        """
-        raise RedisClusterException("method unwatch() is not implemented")
+    async def _unwatch(self, conn):
+        "Unwatches all previously specified keys"
+        await conn.send_command('UNWATCH')
+        res = await conn.read_response()
+        return self.watching and res or True
 
     def script_load_for_pipeline(self, *args, **kwargs):
         """
@@ -701,12 +722,16 @@ class NodeCommands(object):
     """
     """
 
-    def __init__(self, parse_response, connection):
+    def __init__(self, parse_response, connection, in_transaction=False):
         """
         """
         self.parse_response = parse_response
         self.connection = connection
         self.commands = []
+        self.in_transaction = in_transaction
+
+    def extend(self, c):
+        self.commands.extend(c)
 
     def append(self, c):
         """
@@ -734,8 +759,6 @@ class NodeCommands(object):
                 c.result = e
 
     async def read(self):
-        """
-        """
         connection = self.connection
         for c in self.commands:
 
@@ -751,7 +774,13 @@ class NodeCommands(object):
             # explicitly open the connection and all will be well.
             if c.result is None:
                 try:
-                    c.result = await self.parse_response(connection, c.args[0], **c.options)
+                    if self.in_transaction:
+                        cmd = '_'
+                    else:
+                        cmd = c.args[0]
+                    c.result = await self.parse_response(connection, cmd, **c.options)
+                except ExecAbortError:
+                    raise
                 except (ConnectionError, TimeoutError) as e:
                     for c in self.commands:
                         c.result = e
