@@ -77,12 +77,11 @@ class ConnectionPool(object):
 
         Any additional querystring arguments and keyword arguments will be
         passed along to the ConnectionPool class's initializer. The querystring
-        arguments ``socket_connect_timeout`` and ``socket_timeout`` if supplied
-        are parsed as float values. The arguments ``socket_keepalive`` and
-        ``retry_on_timeout`` are parsed to boolean values that accept
-        True/False, Yes/No values to indicate state. Invalid types cause a
-        ``UserWarning`` to be raised. In the case of conflicting arguments,
-        querystring arguments always win.
+        arguments ``connect_timeout`` and ``stream_timeout`` if supplied
+        are parsed as float values. The arguments ``retry_on_timeout`` are
+        parsed to boolean values that accept True/False, Yes/No values to indicate state.
+        Invalid types cause a ``UserWarning`` to be raised.
+        In the case of conflicting arguments, querystring arguments always win.
         """
         url = urlparse(url)
         qs = url.query
@@ -217,7 +216,11 @@ class ConnectionPool(object):
         if connection.pid != self.pid:
             return
         self._in_use_connections.remove(connection)
-        self._available_connections.append(connection)
+        # discard connection with unread response
+        if connection.awaiting_response:
+            connection.disconnect()
+        else:
+            self._available_connections.append(connection)
 
     def disconnect(self):
         "Disconnects all connections in the pool"
@@ -384,7 +387,14 @@ class ClusterConnectionPool(ConnectionPool):
             i_c.remove(connection)
         else:
             pass
-        self._available_connections.setdefault(connection.node["name"], []).append(connection)
+        # discard connection with unread response
+        if connection.awaiting_response:
+            connection.disconnect()
+            # reduce node connection count in case of too many connection error raised
+            if self.max_connections_per_node and self._created_connections_per_node.get(connection.node['name']):
+                self._created_connections_per_node[connection.node['name']] -= 1
+        else:
+            self._available_connections.setdefault(connection.node["name"], []).append(connection)
 
     def disconnect(self):
         """
@@ -412,13 +422,16 @@ class ClusterConnectionPool(ConnectionPool):
         Open new connection to random redis server.
         """
         if self._available_connections:
-            return random.choice(self._available_connections)
-        else:
-            for node in self.nodes.random_startup_node_iter():
-                connection = self.get_connection_by_node(node)
+            node_name = random.choice(list(self._available_connections.keys()))
+            conn_list = self._available_connections[node_name]
+            # check it in case of empty connection list
+            if conn_list:
+                return conn_list.pop()
+        for node in self.nodes.random_startup_node_iter():
+            connection = self.get_connection_by_node(node)
 
-                if connection:
-                    return connection
+            if connection:
+                return connection
 
         raise Exception("Cant reach a single startup node.")
 
