@@ -52,6 +52,15 @@ class TestRedisCommands(object):
         assert isinstance(clients[0], dict)
         assert 'addr' in clients[0]
 
+    @skip_if_server_version_lt('2.6.9')
+    @pytest.mark.asyncio(forbid_global_loop=True)
+    async def test_client_list_after_client_setname(self, r):
+        await r.client_setname('cl=i=ent')
+        clients = await r.client_list()
+        assert isinstance(clients[0], dict)
+        assert 'name' in clients[0]
+        assert clients[-1]['name'] == 'cl=i=ent'
+
     @skip_if_server_version_lt('2.6.9')
     @pytest.mark.asyncio(forbid_global_loop=True)
     async def test_client_getname(self, r):
@@ -773,9 +782,9 @@ class TestRedisCommands(object):
         keys = ['a', 'b', 'c', 'd']
         for index, key in enumerate(keys):
             await r.set(key, index)
-        await r.unlink(keys)
+        await r.unlink(*keys)
         for key in keys:
-            assert r.get(key) is None
+            assert await r.get(key) is None
 
     # LIST COMMANDS
     @pytest.mark.asyncio(forbid_global_loop=True)
@@ -1119,7 +1128,8 @@ class TestRedisCommands(object):
         await r.sadd('a', *s)
         value = await r.spop('a')
         assert set(await r.smembers('a')) == set(s) - set([value])
-    
+
+    @skip_if_server_version_lt('3.2.0')
     @pytest.mark.asyncio(forbid_global_loop=True)
     async def test_spop_multi_value(self, r):
         await r.flushdb()
@@ -1809,6 +1819,12 @@ class TestRedisCommands(object):
         assert await r.geohash('barcelona', 'place1', 'place2') ==\
             [b'sp3e9yg3kd0', b'sp3e9cbc3t0']
 
+    @skip_if_server_version_lt('4.0.0')
+    @pytest.mark.asyncio(forbid_global_loop=True)
+    async def test_geopos_no_value(self, r):
+        await r.flushdb()
+        assert await r.geopos('barcelona', 'place1', 'place2') == [None, None]
+
     @skip_if_server_version_lt('3.2.0')
     @pytest.mark.asyncio(forbid_global_loop=True)
     async def test_geopos(self, r):
